/*
 *	Copyright 1988, Mark Holm
 *			Exceptions
 *
 *	Acknowledgments to Dorothy Robinson for her artistic
 *	 abilities in drawing the icons and to Jim Batch for
 *	 technical support and graphical concepts (which I abandoned in favor
 *       of the easy way out).
 *
 *	Permission is given to copy and distribute for non-profit purposes.
 *
 */

/************************************************************************/
/* Color map definition.                                                */
/************************************************************************/
unsigned char   blue[] = {  0,
                            0,
                            0,
                            0,
                            255,
                            255,
                            255,
                            255 } ,
                green[] = { 0,
                            0,
                            255,
                            255,
                            0,  
                            0,
                            255,
                            255 } ,
                red[] = {   0,
                            255,
                            0,  
                            255,
                            0,  
                            255,
                            0,  
                            255 } ;

#define BLACK	0
#define RED	1
#define GREEN	2
#define YELLOW	3
#define BLUE	4
#define MAGENTA	5
#define CYAN	6
#define WHITE	7

